import urllib.request
import urllib.parse
import json
from urllib.request import Request
from urllib.error import HTTPError

from anylogiccloudclient.client.cloud_error import CloudError


class HttpClient:
    def __init__(self, root_url, api_key):
        self.root_url = root_url
        self.api_key = api_key

    def api_request(self, url, method="GET", body=None):
        try:
            request = Request(self.root_url + urllib.parse.quote(url), method=method, unverifiable=True)
            if body is not None:
                request.data = json.dumps(body).encode('utf-8')
            self._merge_default_headers(request)
            response = urllib.request.urlopen(request)
        except HTTPError as err:
            raise CloudError.from_json(json.loads(err.read()))
        return json.loads(response.read())

    def request(self, url, method="GET", body=None):
        try:
            request = Request(self.root_url + url, method=method, unverifiable=True)
            if body is not None:
                request.data = body
            self._merge_default_headers(request)
            response = urllib.request.urlopen(request)
        except HTTPError as err:
            raise CloudError.from_json(json.loads(err.read()))
        return json.loads(response.read())

    def _merge_default_headers(self, request):
        request.add_header("Authorization", self.api_key)
        request.add_header("Content-Type", "application/json")
