import copy

from anylogiccloudclient.client.utils import case_insensitive_equals, convert_from_string, _convert_to_string
from anylogiccloudclient.client.cloud_error import CloudError
from anylogiccloudclient.client.http_client import HttpClient


class Inputs:
    _replication_nums = "{NUMBER_OF_REPLICATIONS}"

    def __init__(self, http_client: HttpClient, model_version, experiment=None):
        self._http_client = http_client
        self._model_version = model_version
        self.outputs = model_version.experiment_template["outputs"]
        self._set_inputs(experiment)

    def get_input(self, name: str):
        input = self._input_by_name(name)
        return convert_from_string(input["value"], input["type"])

    def set_input(self, name: str, value):  # TODO: serialize value
        input = self._input_by_name(name)
        if input["type"] == "INPUT_FILE":
            file_input = convert_from_string(input["value"], input["type"])
            file_input["hash"] = value
            input["value"] = _convert_to_string(file_input)
        else:
            input["value"] = _convert_to_string(value)

    def set_range_input(self, name, min: int or float, max: int or float, step: int or float):
        input = self._input_by_name(name)
        if input["type"] in ["INTEGER", "FIXED_RANGE_INTEGER"]:
            input["type"] = "FIXED_RANGE_INTEGER"
        elif input["type"] in ["DOUBLE", "FIXED_RANGE_DOUBLE"]:
            input["type"] = "FIXED_RANGE_DOUBLE"
        elif input["type"] in ["LONG", "FIXED_RANGE_LONG"]:
            input["type"] = "FIXED_RANGE_LONG"
        else:
            raise CloudError("Input of type " + input["type"] + " can not vary in range.")
        input["value"] = '{"min":"' + str(min) + '","max":"' + str(max) + '","step":"' + str(step) + '"}'

    def set_number_of_replications(self, num: int):
        try:
            input = self._input_by_name(self._replication_nums)
            input["value"] = num
        except StopIteration:
            self.inputs_array.append({"name": self._replication_nums, "type": "INTEGER", "units": None, "value": num})

    def names(self):
        return list(map(lambda i: i["name"], self.inputs_array))

    def _clone(self):
        new_inputs = Inputs(self._http_client, self._model_version)
        new_inputs.inputs_array = copy.deepcopy(self.inputs_array)
        return new_inputs

    def _set_inputs(self, experiment):
        if experiment is not None:
            self.inputs_array = copy.deepcopy(experiment["inputs"])
        else:
            self.inputs_array = copy.deepcopy(self._model_version.experiment_template["inputs"])
            self.inputs_array.append({"name": "{RANDOM_SEED}", "type": "LONG", "units": None, "value": "1"})

        self.inputs_array = sorted(self.inputs_array, key=lambda i: i["name"])

    def _input_by_name(self, name):
        return next(i for i in self.inputs_array if case_insensitive_equals(i["name"], name))

    def _get_key_data(self, type):
        return {
            "inputs": self.inputs_array,
            "experimentType": type
        }
