import json

from anylogiccloudclient.client import utils
from anylogiccloudclient.client.cloud_error import CloudError
from anylogiccloudclient.data.model_data import ModelData


class MultiRunOutputs:
    def __init__(self, aggregations):
        inputs_aggregation = next(filter(lambda a: len(a.inputs) > 0, aggregations), None)
        if inputs_aggregation != None:
            self._input_names = list(map(lambda i: i.name, inputs_aggregation.inputs))
            self._inputs_list = inputs_aggregation.value
        else:
            self._input_names = []
            self._inputs_list = []

        outputs_aggregations = list(filter(lambda a: len(a.outputs) > 0, aggregations))
        self._outputs = list(map(
            lambda a: ModelData(a.outputs[0].name, a.outputs[0].type, a.value),
            outputs_aggregations
        ))

        self._output_names = list(map(lambda o: o.name, self._outputs))

        self._outputs_list = []
        for i in range(len(self._outputs[0].value)):
            values = []
            for j in range(len(self._outputs)):
                values.append(self._outputs[j].value[i])
            self._outputs_list.append(values)

        self._raw_data = []
        self._raw_data.append(self._input_names + self._output_names)
        for i in range(len(self._inputs_list)):
            self._raw_data.append(self._inputs_list[i] + self._outputs_list[i][0])

    def get_input_names(self):
        return self._input_names

    def get_values_of_input(self, name):
        indexes = [i for i, val in enumerate(self._input_names) if utils.case_insensitive_equals(val, name)]
        if len(indexes) == 0:
            raise CloudError("Input '" + name + "' not found")
        return json.loads(self._inputs_list[indexes[0]])

    def get_output_names(self):
        return self._output_names

    def get_values_of_output(self, name):
        res = list(filter(lambda o: utils.case_insensitive_equals(o.name, name), self._outputs))
        if len(res) == 0:
            raise CloudError("Output '" + name + "' not found")
        return json.loads(*res[0].value)

    def get_raw_data(self):
        return self._raw_data
