import copy

from anylogiccloudclient.client import utils
from anylogiccloudclient.client.cloud_error import CloudError
from anylogiccloudclient.data.file_resource import FileResource


class SingleRunOutputs:
    def __init__(self, aggregations):
        self._outputs = list(map(lambda o: self._identity_to_output(o), aggregations))

    def names(self):
        return list(map(lambda o: o.name, self._outputs))

    def find_name_including(self, name_part: str):
        outputs = list(filter(lambda o: name_part in o.name, self._outputs))
        if len(outputs) == 0:
            raise CloudError("No output names including " + name_part + " part found")
        if len(outputs) >= 2:
            raise CloudError("Multiple output names including " + name_part + " part found")
        return outputs[0].name

    def value(self, name):
        try:
            output = next(filter(lambda o: utils.case_insensitive_equals(o.name, name), self._outputs))
            if output.type == "OUTPUT_FILE":
                file_output = utils.convert_from_string(output.value, output.type)
                return FileResource(file_output["hash"], file_output["localPath"])
            return output.value
        except StopIteration as e:
            raise CloudError("Output value '" + name + "' not found")

    def get_raw_outputs(self):
        return self._outputs

    @staticmethod
    def _identity_to_output(aggregation):
        res = copy.deepcopy(aggregation.outputs[0])
        res.value = aggregation.value[0]
        return res
