import json
import hashlib

from typing import Any
from io import IOBase
from base64 import urlsafe_b64encode
from anylogiccloudclient.client.cloud_error import CloudError


def case_insensitive_equals(name1: str, name2: str) -> bool:
    return name1.strip().lower() == name2.strip().lower()


def convert_from_string(value: str, type_name: str) -> Any:
    if type_name == "STRING":
        return value
    return json.loads(value)


def compute_file_hash(value: str) -> str:
    return _url_safe_sha256_str(value)


def _url_safe_sha256_str(value: bytes) -> str:
    return urlsafe_b64encode(_sha256(value)).decode("utf-8").replace("=", "")


def _sha256(value: bytes) -> bytes:
    return hashlib.sha256(value).digest()


def _convert_to_string(value: Any) -> str:
    if isinstance(value, str):
        return value
    return json.dumps(value)
